package com.kdanmobile.reader

import android.arch.lifecycle.MutableLiveData
import android.arch.lifecycle.ViewModel
import android.content.Context
import android.graphics.Color
import android.net.Uri
import com.kdanmobile.kmpdfkit.annotation.bean.*
import com.kdanmobile.kmpdfkit.globaldata.Config
import com.kdanmobile.kmpdfkit.manager.KMPDFFactory
import com.kdanmobile.kmpdfkit.manager.controller.KMPDFDocumentController
import com.kdanmobile.kmpdfkit.pdfcommon.KMPDFReaderView
import com.kdanmobile.reader.annotationattribute.AnnotationAttribute
import com.kdanmobile.reader.annotationattribute.InkAttribute

class ReaderViewModel(private val pdfSdkLicense: String, private val pdfSdkRsaMsg: String) : ViewModel() {

    enum class ViewDirection(val mode: Config.PDFViewMode) {
        VerticalSinglePageContinues(Config.PDFViewMode.VERTICAL_SINGLE_PAGE_CONTINUES),
        VerticalSinglePage(Config.PDFViewMode.VERTICAL_SINGLE_PAGE),
        HorizontalSinglePage(Config.PDFViewMode.HORIZONTAL_SINGLE_PAGE)
    }

    enum class AnnotationMode {
        None,
        Highlight,
        Strike,
        Underline,
        Ink,
    }

    companion object {
        val DEFAULT_HIGHLIGHT_COLOR = Color.parseColor("#ffdc1b")
        val DEFAULT_STRIKE_COLOR = Color.parseColor("#dd0202")
        val DEFAULT_UNDERLINE_COLOR = Color.parseColor("#dd0202")
        val DEFAULT_INK_COLOR = Color.parseColor("#dd0202")
        const val DEFAULT_HIGHLIGHT_ALPHA = 127
        const val DEFAULT_STRIKE_ALPHA = 255
        const val DEFAULT_UNDERLINE_ALPHA = 255
        const val DEFAULT_INK_ALPHA = 255
        const val DEFAULT_INK_WIDTH = 1f
    }

    var viewDirection = ViewDirection.VerticalSinglePageContinues
        set(value) {
            field = value
            updateViewDirection()
        }
    var isCrop = false
        set(value) {
            field = value
            updateCrop()
        }
    val highLightAttributeLiveData = MutableLiveData<AnnotationAttribute>().apply {
        value = AnnotationAttribute(DEFAULT_HIGHLIGHT_COLOR, DEFAULT_HIGHLIGHT_ALPHA)
    }
    val strikeAttributeLiveData = MutableLiveData<AnnotationAttribute>().apply {
        value = AnnotationAttribute(DEFAULT_STRIKE_COLOR, DEFAULT_STRIKE_ALPHA)
    }
    var underLineAttributeLiveData = MutableLiveData<AnnotationAttribute>().apply {
        value = AnnotationAttribute(DEFAULT_UNDERLINE_COLOR, DEFAULT_UNDERLINE_ALPHA)
    }
    var inkAttributeLiveData = MutableLiveData<InkAttribute>().apply {
        value = InkAttribute(DEFAULT_INK_COLOR, DEFAULT_INK_ALPHA, DEFAULT_INK_WIDTH)
    }

    private var isVerified = false

    private var kmpdfFactory: KMPDFFactory? = null
    private var kmpdfDocumentController: KMPDFDocumentController? = null
    val isOpenedFileLiveData = MutableLiveData<Boolean>().apply { value = false }
    val fileNameLiveData = MutableLiveData<String>()
    val annotationModeLiveData = MutableLiveData<AnnotationMode>().apply { value = AnnotationMode.None }
    var isCopyModeLiveData = MutableLiveData<Boolean>().apply { value = false }

    @JvmOverloads
    fun openPdfFile(context: Context, uri: Uri, type: String? = null): Boolean {
        if (!isVerified) {
            if (!verifyLicense(context)) {
                return false
            }
        }
        return openPdfFIleImp(context, uri, type).apply{
            isOpenedFileLiveData.postValue(this)
        }
    }

    private fun openPdfFIleImp(context: Context, uri: Uri, type: String?): Boolean {
        val factory = KMPDFFactory.open(context, uri, type) ?: return false
        kmpdfFactory = factory
        kmpdfFactory?.also {
            fileNameLiveData.postValue(it.fileName)
            return when {
                it.needPassWord() -> {
                    false
                }
                it.init() -> {
                    true
                }
                else -> {
                    false
                }
            }
        }
        return false
    }

    fun copySelection(): Boolean {
        return kmpdfDocumentController?.copySelection() ?: false
    }

    fun onClickCopyBtn() {
        if (annotationModeLiveData.value != AnnotationMode.None) stopAnnotationMode()
        if (isCopyModeLiveData.value == true) {
            stopCopyTextMode()
        } else {
            startCopyTextMode()
        }
    }

    fun onClickHighlightBtn() {
        if (isCopyModeLiveData.value == true) stopCopyTextMode()
        if (annotationModeLiveData.value == AnnotationMode.Highlight) {
            stopAnnotationMode()
        } else {
            startHighLightEditMode()
        }
    }

    fun onLongClickHighlightBtn() {
        if (isCopyModeLiveData.value == true) stopCopyTextMode()
        startHighLightEditMode()
    }

    fun onClickStrikeBtn() {
        if (isCopyModeLiveData.value == true) stopCopyTextMode()
        if (annotationModeLiveData.value == AnnotationMode.Strike) {
            stopAnnotationMode()
        } else {
            startStrikeOutEditMode()
        }
    }

    fun onLongClickStrikeBtn() {
        if (isCopyModeLiveData.value == true) stopCopyTextMode()
        startStrikeOutEditMode()
    }

    fun onClickUnderlineBtn() {
        if (isCopyModeLiveData.value == true) stopCopyTextMode()
        if (annotationModeLiveData.value == AnnotationMode.Underline) {
            stopAnnotationMode()
        } else {
            startUnderLineEditMode()
        }
    }

    fun onLongClickUnderlinBtn() {
        if (isCopyModeLiveData.value == true) stopCopyTextMode()
        startUnderLineEditMode()
    }

    fun onClickInkBtn() {
        if (isCopyModeLiveData.value == true) stopCopyTextMode()
        if (annotationModeLiveData.value == AnnotationMode.Ink) {
            stopAnnotationMode()
        } else {
            startInkEditMode()
        }
    }

    fun onLongClickInkBtn() {
        if (isCopyModeLiveData.value == true) stopCopyTextMode()
        startInkEditMode()
    }

    private fun startCopyTextMode() {
        kmpdfDocumentController?.startCopyText()
        isCopyModeLiveData.postValue(true)
    }

    private fun stopCopyTextMode() {
        kmpdfDocumentController?.stopCopyText()
        isCopyModeLiveData.postValue(false)
    }

    fun setHighLightAttributes(color: Int, alpha: Int) {
        highLightAttributeLiveData.postValue(AnnotationAttribute(color, alpha))
        kmpdfFactory?.setAnnotationAttribute(KMPDFHighlightAnnotationBean("", color, alpha))
        kmpdfFactory?.annotConfig?.apply {
            markerPenColor_hightlight = color
            markerPenAlpha_hightlight = alpha
        }
    }

    private fun startHighLightEditMode() {
        kmpdfFactory?.setAnnotationEditMode(KMPDFAnnotationBean.AnnotationType.HIGH_LIGHT)
        annotationModeLiveData.postValue(AnnotationMode.Highlight)
    }

    fun setStrikeOutAttributes(color: Int, alpha: Int) {
        strikeAttributeLiveData.postValue(AnnotationAttribute(color, alpha))
        kmpdfFactory?.setAnnotationAttribute(KMPDFStrikeoutAnnotationBean("", color, alpha))
        kmpdfFactory?.annotConfig?.apply {
            markerPenColor_strikeout = color
            markerPenAlpha_strikeout = alpha
        }
    }

    private fun startStrikeOutEditMode() {
        kmpdfFactory?.setAnnotationEditMode(KMPDFAnnotationBean.AnnotationType.STRIKE_OUT)
        annotationModeLiveData.postValue(AnnotationMode.Strike)
    }

    fun setUnderLineAttributes(color: Int, alpha: Int) {
        underLineAttributeLiveData.postValue(AnnotationAttribute(color, alpha))
        kmpdfFactory?.setAnnotationAttribute(KMPDFUnderlineAnnotationBean("", color, alpha))
        kmpdfFactory?.annotConfig?.apply {
            markerPenColor_underline = color
            markerPenAlpha_underline = alpha
        }
    }

    private fun startUnderLineEditMode() {
        kmpdfFactory?.setAnnotationEditMode(KMPDFAnnotationBean.AnnotationType.UNDER_LINE)
        annotationModeLiveData.postValue(AnnotationMode.Underline)
    }

    fun setInkAttributes(color: Int, alpha: Int, width: Float) {
        inkAttributeLiveData.postValue(InkAttribute(color, alpha, width))
        kmpdfFactory?.apply {
            setAnnotationAttribute(KMPDFInkAnnotationBean("", color, width, alpha))
            annotConfig?.apply {
                markerPenColor_ink = color
                markerPenAlpha_ink = alpha
                markerPenSize_ink = width
            }
        }
    }

    private fun startInkEditMode() {
        kmpdfFactory?.setAnnotationEditMode(KMPDFAnnotationBean.AnnotationType.INK)
        annotationModeLiveData.postValue(AnnotationMode.Ink)
    }

    private fun stopAnnotationMode() {
        kmpdfFactory?.setAnnotationEditMode(KMPDFAnnotationBean.AnnotationType.NULL)
        annotationModeLiveData.postValue(AnnotationMode.None)
    }

    fun setReaderView(readerView: KMPDFReaderView) {
        kmpdfFactory?.setReaderView(readerView)
        kmpdfDocumentController = kmpdfFactory?.getController(KMPDFFactory.ControllerType.DOCUMENT) as KMPDFDocumentController
        updateViewDirection()
    }

    private fun verifyLicense(context: Context): Boolean {
        val errorCode = KMPDFFactory.verifyLicense(context, pdfSdkLicense, pdfSdkRsaMsg)
        return errorCode == 0
    }

    private fun updateViewDirection() {
        kmpdfDocumentController?.pdfViewMode = viewDirection.mode
        kmpdfDocumentController?.refresh(false)
    }

    private fun updateCrop() {
        kmpdfDocumentController?.apply {
            cropMode = isCrop
            refresh(false)
        }
    }
}